@@ -6,6 +6,7 @@ from .exceptions import RequestValidationError
 from .exceptions import ResponseValidationError
 from .wrappers import PyramidOpenAPIRequestFactory
 from openapi_core import create_spec
+from openapi_core.schema.specs.models import Spec
 from openapi_core.validation.exceptions import InvalidSecurity
 from openapi_core.validation.request.validators import RequestValidator
 from openapi_core.validation.response.validators import ResponseValidator
@@ -14,6 +15,7 @@ from openapi_spec_validator.schemas import read_yaml_file
 from pathlib import Path
 from pyramid.config import Configurator
 from pyramid.config import PHASE0_CONFIG
+from pyramid.config import PHASE1_CONFIG
 from pyramid.config.views import ViewDeriverInfo
 from pyramid.events import ApplicationCreated
 from pyramid.exceptions import ConfigurationError
@@ -34,16 +36,16 @@ import typing as t
 
 logger = logging.getLogger(__name__)
 
-APIS = []
-
 
 def includeme(config: Configurator) -> None:
     """Pyramid knob."""
+    config.add_request_method(openapi_validated, name="openapi_validated", reify=True)
     config.add_view_deriver(openapi_view)
     config.add_directive("pyramid_openapi3_add_formatter", add_formatter)
     config.add_directive("pyramid_openapi3_add_explorer", add_explorer_view)
     config.add_directive("pyramid_openapi3_spec", add_spec_view)
     config.add_directive("pyramid_openapi3_spec_directory", add_spec_view_directory)
+    config.add_directive("pyramid_openapi3_register_routes", register_routes)
     config.add_tween("pyramid_openapi3.tween.response_tween_factory", over=EXCVIEW)
     config.add_subscriber(check_all_routes, ApplicationCreated)
 
@@ -61,6 +63,34 @@ def includeme(config: Configurator) -> None:
     )
 
 
+def openapi_validated(request: Request) -> dict:
+    """Get validated parameters."""
+    # Validate request and attach all findings for view to introspect
+    validate_request = asbool(
+        request.registry.settings.get(
+            "pyramid_openapi3.enable_request_validation", True
+        )
+    )
+    validate_response = asbool(
+        request.registry.settings.get(
+            "pyramid_openapi3.enable_response_validation", True
+        )
+    )
+    request.environ["pyramid_openapi3.validate_response"] = validate_response
+    gsettings = settings = request.registry.settings["pyramid_openapi3"]
+    route_settings = gsettings.get("routes")
+    if route_settings and request.matched_route.name in route_settings:
+        settings = request.registry.settings[route_settings[request.matched_route.name]]
+
+    if validate_request:  # pragma: no branch
+        request.environ["pyramid_openapi3.validate_request"] = True
+        openapi_request = PyramidOpenAPIRequestFactory.create(request)
+        validated = settings["request_validator"].validate(openapi_request)
+        return validated
+
+    return {}  # pragma: no cover
+
+
 Context = t.TypeVar("Context")
 View = t.Callable[[Context, Request], Response]
 
@@ -78,39 +108,13 @@ def openapi_view(view: View, info: ViewDeriverInfo) -> View:
     if info.options.get("openapi"):
 
         def wrapper_view(context: Context, request: Request) -> Response:
-            # Validate request and attach all findings for view to introspect
             validate_request = asbool(
                 request.registry.settings.get(
                     "pyramid_openapi3.enable_request_validation", True
                 )
             )
-            validate_response = asbool(
-                request.registry.settings.get(
-                    "pyramid_openapi3.enable_response_validation", True
-                )
-            )
-            request.environ["pyramid_openapi3.validate_response"] = validate_response
-            gsettings = settings = request.registry.settings["pyramid_openapi3"]
-            if "routes" in gsettings:
-                settings = request.registry.settings[
-                    gsettings["routes"][request.matched_route.name]
-                ]
-
-            # Needed to support relative `servers` entries in `openapi.yaml`,
-            # see https://github.com/p1c2u/openapi-core/issues/218.
-            settings["request_validator"].base_url = request.application_url
-            settings["response_validator"].base_url = request.application_url
-
-            if validate_request:
-                request.environ["pyramid_openapi3.validate_request"] = True
-                openapi_request = PyramidOpenAPIRequestFactory.create(request)
-                request.openapi_validated = settings["request_validator"].validate(
-                    openapi_request
-                )
-                if request.openapi_validated.errors:
-                    raise RequestValidationError(
-                        errors=request.openapi_validated.errors
-                    )
+            if validate_request and request.openapi_validated.errors:
+                raise RequestValidationError(errors=request.openapi_validated.errors)
 
             # Do the view
             return view(context, request)
@@ -147,7 +151,8 @@ def add_explorer_view(
             settings = config.registry.settings
             if settings.get(apiname) is None:
                 raise ConfigurationError(
-                    "You need to call config.pyramid_openapi3_spec for explorer to work."
+                    "You need to call config.pyramid_openapi3_spec for the explorer "
+                    "to work."
                 )
             with open(resolved_template.abspath()) as f:
                 template = Template(f.read())
@@ -222,7 +227,9 @@ def add_spec_view(
                 spec, custom_formatters=custom_formatters
             ),
         }
-        APIS.append(apiname)
+        config.registry.settings.setdefault("pyramid_openapi3_apinames", []).append(
+            apiname
+        )
 
     config.action((f"{apiname}_spec",), register, order=PHASE0_CONFIG)
 
@@ -281,6 +288,33 @@ def add_spec_view_directory(
     config.action(("pyramid_openapi3_spec",), register, order=PHASE0_CONFIG)
 
 
+def register_routes(
+    config: Configurator,
+    route_name_ext: str = "x-pyramid-route-name",
+    root_factory_ext: str = "x-pyramid-root-factory",
+    apiname: str = "pyramid_openapi3",
+) -> None:
+    """Register routes to app from OpenApi 3.0 specification.
+
+    :param route_name_ext: Extension's key for using a ``route_name`` argument
+    :param root_factory_ext: Extension's key for using a ``factory`` argument
+    """
+
+    def action() -> None:
+        spec = config.registry.settings[apiname]["spec"]
+        for pattern, path_item in spec.paths.items():
+            route_name = path_item.extensions.get(route_name_ext)
+            if route_name:
+                root_factory = path_item.extensions.get(root_factory_ext)
+                config.add_route(
+                    route_name.value,
+                    pattern=pattern,
+                    factory=root_factory.value if root_factory else None,
+                )
+
+    config.action(("pyramid_openapi3_register_routes",), action, order=PHASE1_CONFIG)
+
+
 def openapi_validation_error(
     context: t.Union[RequestValidationError, ResponseValidationError], request: Request
 ) -> Response:
@@ -316,43 +350,36 @@ def check_all_routes(event: ApplicationCreated):
 
     app = event.app
     settings = app.registry.settings
-    for name in APIS:
-        openapi_settings = settings.get(name)
-        if not openapi_settings:
-            # pyramid_openapi3 not configured?
-            logger.warning(
-                "pyramid_openapi3 settings not found. "
-                "Did you forget to call config.pyramid_openapi3_spec?"
-            )
-            return
+    apinames = settings.get("pyramid_openapi3_apinames")
+    if not apinames:
+        # pyramid_openapi3 not configured?
+        logger.warning(
+            "pyramid_openapi3 settings not found. "
+            "Did you forget to call config.pyramid_openapi3_spec?"
+        )
+        return
+
+    for name in apinames:  # pragma: no branch
+        openapi_settings = settings[name]
 
         if not settings.get("pyramid_openapi3.enable_endpoint_validation", True):
             logger.info("Endpoint validation against specification is disabled")
             return
 
-        # Sometimes api routes are prefixed with `/api/v1` and similar, using
-        # https://swagger.io/docs/specification/api-host-and-base-path/
-        prefixes = []
-        for server in openapi_settings["spec"].servers:
-            path = urlparse(server.url).path
-            if path != "/":
-                prefixes.append(path)
+        prefixes = _get_server_prefixes(openapi_settings["spec"])
 
         def remove_prefixes(path):
             path = f"/{path}" if not path.startswith("/") else path
             for prefix in prefixes:
-                path = path.replace(prefix, "")
+                if path.startswith(prefix):
+                    prefix_length = len(prefix)
+                    return path[prefix_length:]
             return path
 
         paths = list(openapi_settings["spec"].paths.keys())
         routes = [
-            remove_prefixes(route.path)
-            for name, route in app.routes_mapper.routes.items()
+            remove_prefixes(route.path) for route in app.routes_mapper.routes.values()
         ]
-        route_names = {
-            remove_prefixes(route.path): name
-            for name, route in app.routes_mapper.routes.items()
-        }
 
         missing = [r for r in paths if r not in routes]
         if missing:
@@ -360,5 +387,25 @@ def check_all_routes(event: ApplicationCreated):
 
         settings.setdefault("pyramid_openapi3", {})
         settings["pyramid_openapi3"].setdefault("routes", {})
-        for path in paths:
-            settings["pyramid_openapi3"]["routes"][route_names[path]] = name
+
+        # It is possible to have multiple `add_route` for a single path
+        # (due to request_method predicates). So loop through each route
+        # to create a lookup of route_name -> api_name
+        for route_name, route in app.routes_mapper.routes.items():
+            if remove_prefixes(route.path) in paths:
+                settings["pyramid_openapi3"]["routes"][route_name] = name
+
+
+def _get_server_prefixes(spec: Spec) -> t.List[str]:
+    """Build a set of possible route prefixes from the api spec.
+
+    Api routes may optionally be prefixed using servers (e.g: `/api/v1`).
+    See: https://swagger.io/docs/specification/api-host-and-base-path/
+    """
+    prefixes = []
+    for server in spec.servers:
+        path = urlparse(server.url).path
+        path = f"/{path}" if not path.startswith("/") else path
+        if path != "/":
+            prefixes.append(path)
+    return prefixes
